/*

Location: Not stored (in-memory only)
Location scope: Local (only accessible from CTE chain or final query)
Time scope: Exist until execution completes
Indexable: No
Copy of data: No
Modifiable: Yes, modifies underlying data

*/

-- Basic CTE Usage

/*
with JK_CTE as (
select myIntKey
     , myVarchar
     , '42' as [The Answer]
  from dbo.JK_Temp_Stuff_Test_Table
 union all
select myIntKey
     , myVarchar
     , '42' as [The Answer]
  from dbo.JK_Temp_Stuff_Test_Table
)

select *
  from JK_CTE;
*/

-- CTE Goes out of scope

/*
with JK_CTE as (
select myIntKey
     , myVarchar
     , '42' as [The Answer]
  from dbo.JK_Temp_Stuff_Test_Table
 union all
select myIntKey
     , myVarchar
     , '42' as [The Answer]
  from dbo.JK_Temp_Stuff_Test_Table
)

select *
  from JK_CTE;

select *
  from JK_CTE;
*/

-- CTE Chains

/*
with JK_CTE as (
select myIntKey
     , myVarchar
     , '42' as [The Answer]
  from dbo.JK_Temp_Stuff_Test_Table
),

JK_CTE_2 as (
select *
  from JK_CTE
 union all
select *
  from JK_CTE
)

select *
  from JK_CTE_2;
*/

-- Recursive CTEs

/*
with JK_CTE as (
select myIntKey
     , myVarchar
     , '42' as [The Answer]
  from dbo.JK_Temp_Stuff_Test_Table
),

JK_CTE_2 as (
select *
  from JK_CTE
 union all
select myIntKey + 1
     , myVarchar
     , [The Answer]
  from JK_CTE_2
 where myIntKey <= 42
)

select *
  from JK_CTE_2;
*/

-- Scope of CTE references

/*
with JK_CTE as (
select myIntKey
     , myVarchar
     , '42' as [The Answer]
  from dbo.JK_Temp_Stuff_Test_Table
),

JK_CTE_2 as (
select *
  from dbo.PROPERTY_VEHICLE_HISTORY
)

select *
  from JK_CTE;
*/

-- CTEs want semicolons

/*
select * from dbo.JK_Temp_Stuff_Test_Table

with JK_CTE as (
select myIntKey
     , myVarchar
     , '42' as [The Answer]
  from dbo.JK_Temp_Stuff_Test_Table
)

select *
  from JK_CTE;
*/

-- CTEs update the underlying table

/*
with JK_CTE as (
     select *
       from dbo.JK_Temp_Stuff_Test_Table
)

update JK_CTE
   set myVarchar = 'CTE Test';

select *
  from dbo.JK_Temp_Stuff_Test_Table;
*/

-- Cleanup

/*
update dbo.JK_Temp_Stuff_Test_Table set myVarchar = 'Wheeee';
*/

